@@ -9,8 +9,8 @@ autoconf
 
 echo "
 suggested configure parameters:
-# for windrbd (on a cygwin host, assuming C:\windrbd is your windrbd root)
-./configure --prefix=/cygdrive/c/windrbd/usr --localstatedir=/cygdrive/c/windrbd/var --sysconfdir=/cygdrive/c/windrbd/etc --without-83support --without-84support --without-drbdmon --with-windrbd
+# for windrbd (on a cygwin host)
+./configure --without-83support --without-84support --without-drbdmon --with-windrbd --without-manual
 # prepare for rpmbuild, only generate spec files
 ./configure --enable-spec
 # or prepare for direct build
