@@ -5,6 +5,7 @@
 #include <linux/types.h>
 #else
 #include <stdint.h>
+#define u64 uint64_t
 #endif
 
 #ifndef ARRAY_SIZE
@@ -103,6 +104,7 @@ enum drbd_packet {
 	P_RS_CANCEL_AHEAD     = 0x4a, /* protocol version 115,
 		 * meta: cancel RS_DATA_REQUEST packet if already Ahead again,
 		 *       tell peer to stop sending resync requests... */
+	P_DISCONNECT          = 0x4b, /* data sock: Disconnect and stop connection attempts */
 
 	P_MAY_IGNORE	      = 0x100, /* Flag to test if (cmd > P_MAY_IGNORE) ... */
 
@@ -307,7 +309,7 @@ struct p_rs_param {
 	uint32_t resync_rate;
 
 	      /* Since protocol version 88 and higher. */
-	char verify_alg[0];
+	char verify_alg[];
 } __packed;
 
 struct p_rs_param_89 {
@@ -341,14 +343,14 @@ struct p_protocol {
 	uint32_t two_primaries;
 
               /* Since protocol version 87 and higher. */
-	char integrity_alg[0];
+	char integrity_alg[];
 
 } __packed;
 
-#define UUID_FLAG_DISCARD_MY_DATA 1
-#define UUID_FLAG_CRASHED_PRIMARY 2
-#define UUID_FLAG_INCONSISTENT 4
-#define UUID_FLAG_SKIP_INITIAL_SYNC 8
+#define UUID_FLAG_DISCARD_MY_DATA     ((u64)1 << 0)
+#define UUID_FLAG_CRASHED_PRIMARY     ((u64)1 << 1)
+#define UUID_FLAG_INCONSISTENT        ((u64)1 << 2)
+#define UUID_FLAG_SKIP_INITIAL_SYNC   ((u64)1 << 3)
 
 #define UUID_FLAG_MASK_COMPAT_84 \
 	(UUID_FLAG_DISCARD_MY_DATA|\
@@ -356,14 +358,18 @@ struct p_protocol {
 	 UUID_FLAG_INCONSISTENT|\
 	 UUID_FLAG_SKIP_INITIAL_SYNC)
 
-#define UUID_FLAG_NEW_DATAGEN 16
-#define UUID_FLAG_STABLE 32
-#define UUID_FLAG_GOT_STABLE 64 /* send UUIDs */
-#define UUID_FLAG_RESYNC 128    /* compare UUIDs and eventually start resync */
-#define UUID_FLAG_RECONNECT 256
-#define UUID_FLAG_DISKLESS_PRIMARY 512 /* Use with UUID_FLAG_RESYNC if a diskless primary is
-					  the reason */
-#define UUID_FLAG_PRIMARY_LOST_QUORUM 1024
+#define UUID_FLAG_NEW_DATAGEN         ((u64)1 << 4)
+#define UUID_FLAG_STABLE              ((u64)1 << 5)
+#define UUID_FLAG_GOT_STABLE          ((u64)1 << 6) /* send UUIDs */
+#define UUID_FLAG_RESYNC              ((u64)1 << 7) /* compare UUIDs and eventually start resync */
+#define UUID_FLAG_RECONNECT           ((u64)1 << 8)
+#define UUID_FLAG_DISKLESS_PRIMARY    ((u64)1 << 9) /* Use with UUID_FLAG_RESYNC if a diskless primary is the reason */
+#define UUID_FLAG_PRIMARY_LOST_QUORUM ((u64)1 << 10)
+#define UUID_FLAG_SYNC_TARGET         ((u64)1 << 11) /* currently L_SYNC_TARGET to some peer */
+#define UUID_FLAG_HAS_UNALLOC         ((u64)1 << 12) /* highest byte contains index of not allocated bitmap uuid */
+
+#define UUID_FLAG_UNALLOC_SHIFT       56
+#define UUID_FLAG_UNALLOC_MASK        ((u64)0xff << UUID_FLAG_UNALLOC_SHIFT)
 
 struct p_uuids {
 	uint64_t current_uuid;
@@ -381,7 +387,7 @@ struct p_uuids110 {
 			       authoritative nodes when UUID_FLAG_STABLE not set */
 
 	uint64_t bitmap_uuids_mask; /* non zero bitmap UUIDS for these nodes */
-	uint64_t other_uuids[0]; /* the first hweight(bitmap_uuids_mask) slots carry bitmap uuids.
+	uint64_t other_uuids[]; /* the first hweight(bitmap_uuids_mask) slots carry bitmap uuids.
 				    The node with the lowest node_id first.
 				    The remaining slots carry history uuids */
 } __packed;
@@ -438,7 +444,7 @@ struct p_sizes {
 	uint16_t dds_flags; /* use enum dds_flags here. */
 
 	/* optional queue_limits if (agreed_features & DRBD_FF_WSAME) */
-	struct o_qlim qlim[0];
+	struct o_qlim qlim[];
 } __packed;
 
 struct p_state {
@@ -528,7 +534,7 @@ struct p_compressed_bm {
 	 */
 	uint8_t encoding;
 
-	uint8_t code[0];
+	uint8_t code[];
 } __packed;
 
 struct p_delay_probe93 {
