@@ -133,6 +133,7 @@ static int do_proxy_conn_plugins(const struct cfg_ctx *ctx);
 
 int ctx_by_name(struct cfg_ctx *ctx, const char *id, checks check);
 int was_file_already_seen(char *fn);
+bool is_set_gi_single_node(const struct cfg_ctx *ctx);
 
 static char *get_opt_val(struct options *, const char *, char *);
 
@@ -342,8 +343,8 @@ static struct adm_cmd up_cmd = {"up", adm_up, ACF1_RESNAME_VERIFY_IPS };
 static struct adm_cmd down_cmd = {"down", adm_drbdsetup, ACF1_RESNAME .takes_long = 1};
 static struct adm_cmd primary_cmd = {"primary", adm_drbdsetup, &primary_cmd_ctx, ACF1_RESNAME .takes_long = 1};
 static struct adm_cmd secondary_cmd = {"secondary", adm_drbdsetup, ACF1_RESNAME .takes_long = 1};
-static struct adm_cmd invalidate_cmd = {"invalidate", adm_invalidate, ACF1_MINOR_ONLY };
-static struct adm_cmd invalidate_remote_cmd = {"invalidate-remote", adm_drbdsetup, ACF1_PEER_DEVICE .takes_long = 1};
+static struct adm_cmd invalidate_cmd = {"invalidate", adm_invalidate, &invalidate_ctx, ACF1_MINOR_ONLY };
+static struct adm_cmd invalidate_remote_cmd = {"invalidate-remote", adm_drbdsetup, &invalidate_peer_ctx, ACF1_PEER_DEVICE .takes_long = 1};
 static struct adm_cmd outdate_cmd = {"outdate", adm_outdate, ACF1_DEFAULT};
 /*  */ struct adm_cmd resize_cmd = {"resize", adm_resize, &resize_cmd_ctx, ACF1_DEFAULT .disk_required = 1};
 static struct adm_cmd verify_cmd = {"verify", adm_drbdsetup, &verify_cmd_ctx, ACF1_PEER_DEVICE};
@@ -356,7 +357,7 @@ static struct adm_cmd wait_sync_cmd = {"wait-sync", adm_wait_c, ACF1_WAIT};
 static struct adm_cmd wait_ci_cmd = {"wait-con-int", adm_wait_ci, .show_in_usage = 1,.verify_ips = 1,};
 static struct adm_cmd role_cmd = {"role", adm_drbdsetup, ACF1_RESNAME};
 static struct adm_cmd cstate_cmd = {"cstate", adm_drbdsetup, ACF1_DISCONNECT};
-static struct adm_cmd dstate_cmd = {"dstate", adm_setup_and_meta, ACF1_MINOR_ONLY };
+static struct adm_cmd dstate_cmd = {"dstate", adm_setup_and_meta, ACF1_MINOR_ONLY .disk_required = 0 };
 static struct adm_cmd status_cmd = {"status", adm_drbdsetup, .show_in_usage = 1, .uc_dialog = 1, .backend_res_name=1};
 static struct adm_cmd peer_device_options_cmd = {"peer-device-options", adm_peer_device,
 						 &peer_device_options_ctx, ACF1_PEER_DEVICE};
@@ -380,7 +381,7 @@ static struct adm_cmd sh_mod_parms_cmd = {"sh-mod-parms", sh_mod_parms, ACF2_GEN
 static struct adm_cmd sh_dev_cmd = {"sh-dev", sh_dev, ACF2_SHELL};
 static struct adm_cmd sh_udev_cmd = {"sh-udev", sh_udev, .vol_id_required = 1, ACF2_HOOK};
 static struct adm_cmd sh_minor_cmd = {"sh-minor", sh_minor, ACF2_SHELL};
-static struct adm_cmd sh_ll_dev_cmd = {"sh-ll-dev", sh_ll_dev, ACF2_SHELL .disk_required = 1};
+static struct adm_cmd sh_ll_dev_cmd = {"sh-ll-dev", sh_ll_dev, ACF2_SHELL .disk_required = 0};
 static struct adm_cmd sh_md_dev_cmd = {"sh-md-dev", sh_md_dev, ACF2_SHELL .disk_required = 1};
 static struct adm_cmd sh_md_idx_cmd = {"sh-md-idx", sh_md_idx, ACF2_SHELL .disk_required = 1};
 static struct adm_cmd sh_ip_cmd = {"sh-ip", sh_ip, ACF2_SHELL};
@@ -543,6 +544,7 @@ struct adm_cmd *cmds[] = {
 static const struct adm_cmd invalidate_setup_cmd = {
 	"invalidate",
 	__adm_drbdsetup_silent,
+	&invalidate_ctx,
 	ACF1_MINOR_ONLY
 };
 
@@ -683,13 +685,18 @@ int call_cmd(const struct adm_cmd *cmd, const struct cfg_ctx *ctx,
 				break;
 		}
 	} else if (iterate_conns) {
-		for_each_connection(conn, &res->connections) {
-			if (conn->ignore)
-				continue;
-			tmp_ctx.conn = conn;
+		if (is_set_gi_single_node(&tmp_ctx))
 			ret = __call_cmd_fn(&tmp_ctx, on_error);
-			if (ret)
-				break;
+		else {
+			for_each_connection(conn, &res->connections) {
+				if (conn->ignore) {
+					continue;
+				}
+				tmp_ctx.conn = conn;
+				ret = __call_cmd_fn(&tmp_ctx, on_error);
+				if (ret)
+					break;
+			}
 		}
 	} else {
 		ret = __call_cmd_fn(&tmp_ctx, on_error);
@@ -918,9 +925,16 @@ static int sh_lres(const struct cfg_ctx *ctx)
 	return 0;
 }
 
+/* returns either the backing dev path or "none" if it is empty */
+static const char * const backing_disk_str(struct d_volume *info) {
+	if (!info || !info->disk)
+		return "none";
+	return info->disk;
+}
+
 static int sh_ll_dev(const struct cfg_ctx *ctx)
 {
-	printf("%s\n", ctx->vol->disk);
+	printf("%s\n", backing_disk_str(ctx->vol));
 	return 0;
 }
 
@@ -1097,7 +1111,7 @@ static void add_setup_options(char **argv, int *argcp, const struct context_def
 	*argcp = argc;
 }
 
-#define make_option(ARG, OPT) do {						\
+#define make_option(ARG, OPT, OPTIONS_DEF) do {					\
 	struct d_name *b_opt; 							\
 	bool found = false; 							\
 	STAILQ_FOREACH(b_opt, &backend_options, link) {				\
@@ -1107,17 +1121,23 @@ static void add_setup_options(char **argv, int *argcp, const struct context_def
 		} 								\
 	} 									\
 	if (!found) {								\
-		if(OPT->value)							\
-			ARG = ssprintf("--%s=%s", OPT->name, OPT->value);	\
-		else  								\
+		if(OPT->value) {						\
+			int w = 512;						\
+			const struct field_def *field =				\
+				find_field(NULL, OPTIONS_DEF, OPT->name);	\
+			if (field && field->ops == &fc_string)			\
+				w = field->u.s.max_len;				\
+			ARG = ssprintf("--%s=%.*s", OPT->name, w, OPT->value);	\
+		} else {							\
 			ARG = ssprintf("--%s", OPT->name);			\
+		}								\
 	}									\
 } while (0)
 
-#define make_options(ARG, OPTIONS) do {					\
+#define make_options(ARG, OPTIONS, OPTIONS_DEF) do {			\
 	struct d_option *option;					\
 	STAILQ_FOREACH(option, OPTIONS, link) 				\
-		make_option(ARG, option);				\
+		make_option(ARG, option, OPTIONS_DEF);			\
 } while (0)
 
 #define ssprintf_addr(A)					\
@@ -1162,9 +1182,9 @@ static int adm_attach(const struct cfg_ctx *ctx)
 		argv[NA(argc)] = "--set-defaults";
 		STAILQ_FOREACH(option, &ctx->vol->disk_options, link)
 			if (!option->adj_skip)
-				make_option(argv[NA(argc)], option);
+				make_option(argv[NA(argc)], option, ctx->cmd->drbdsetup_ctx);
 	} else
-		make_options(argv[NA(argc)], &ctx->vol->disk_options);
+		make_options(argv[NA(argc)], &ctx->vol->disk_options, ctx->cmd->drbdsetup_ctx);
 	add_setup_options(argv, &argc, ctx->cmd->drbdsetup_ctx);
 	argv[NA(argc)] = 0;
 
@@ -1235,7 +1255,7 @@ static int adm_resource(const struct cfg_ctx *ctx)
 	if (reset)
 		argv[NA(argc)] = "--set-defaults";
 	if (reset || do_new_resource)
-		make_options(argv[NA(argc)], &res->res_options);
+		make_options(argv[NA(argc)], &res->res_options, ctx->cmd->drbdsetup_ctx);
 	add_setup_options(argv, &argc, ctx->cmd->drbdsetup_ctx);
 	argv[NA(argc)] = NULL;
 
@@ -1353,6 +1373,20 @@ int adm_resize(const struct cfg_ctx *ctx)
 	return 0;
 }
 
+bool is_set_gi_single_node(const struct cfg_ctx *ctx)
+{
+	if (ctx->cmd == &set_gi_cmd) {
+		int nr_hosts = 0;
+		struct d_host_info *host;
+		for_each_host(host, &ctx->res->all_hosts)
+			nr_hosts++;
+		if (nr_hosts == 1)
+			return true;
+	}
+
+	return false;
+}
+
 int _adm_drbdmeta(const struct cfg_ctx *ctx, int flags, char *argument)
 {
 	struct d_volume *vol = ctx->vol;
@@ -1378,8 +1412,12 @@ int _adm_drbdmeta(const struct cfg_ctx *ctx, int flags, char *argument)
 	} else {
 		argv[NA(argc)] = vol->meta_index;
 	}
-	if (ctx->cmd->need_peer)
-		argv[NA(argc)] = ssprintf("--node-id=%s", ctx->conn->peer->node_id);
+	if (ctx->cmd->need_peer) {
+		if (is_set_gi_single_node(ctx))
+			argv[NA(argc)] = ssprintf("--node-id=%s", ctx->res->me->node_id);
+		else
+			argv[NA(argc)] = ssprintf("--node-id=%s", ctx->conn->peer->node_id);
+	}
 	argv[NA(argc)] = (char *)ctx->cmd->name;
 	if (argument)
 		argv[NA(argc)] = argument;
@@ -1531,13 +1569,20 @@ static int adm_chk_resize(const struct cfg_ctx *ctx)
 	return adm_drbdmeta(ctx);
 }
 
+bool backing_device_configured(const struct cfg_ctx *ctx)
+{
+	return (ctx->vol && ctx->vol->disk && ctx->vol->meta_disk);
+}
+
+/* try drbdsetup first (i.e., what does the kernel know
+ * if it fails, try drbdmeta */
 static int adm_setup_and_meta(const struct cfg_ctx *ctx)
 {
 	int rv;
 
 	rv = __adm_drbdsetup_silent(ctx);
 
-	if (rv == 11 || rv == 17) {
+	if (rv == 11 || rv == 17 || !backing_device_configured(ctx)) {
 		/* see drbdsetup.c, print_config_error():
 		 *  11: some unspecific state change error. (ignore for invalidate)
 		 *  17: SS_NO_UP_TO_DATE_DISK */
@@ -1730,7 +1775,7 @@ int adm_peer_device(const struct cfg_ctx *ctx)
 	if (reset)
 		argv[NA(argc)] = "--set-defaults";
 
-	make_options(argv[NA(argc)], &peer_device->pd_options);
+	make_options(argv[NA(argc)], &peer_device->pd_options, ctx->cmd->drbdsetup_ctx);
 	add_setup_options(argv, &argc, ctx->cmd->drbdsetup_ctx);
 	argv[NA(argc)] = 0;
 
@@ -1776,7 +1821,7 @@ static int adm_new_peer(const struct cfg_ctx *ctx)
 	if (!strncmp(ctx->cmd->name, "net-options", 11))
 		del_opt(&conn->net_options, "transport");
 
-	make_options(argv[NA(argc)], &conn->net_options);
+	make_options(argv[NA(argc)], &conn->net_options, ctx->cmd->drbdsetup_ctx);
 
 	add_setup_options(argv, &argc, ctx->cmd->drbdsetup_ctx);
 	argv[NA(argc)] = 0;
@@ -2067,7 +2112,7 @@ static int adm_wait_c(const struct cfg_ctx *ctx)
 		argv[argc] = ssprintf("%lu", timeout);
 		argc++;
 	} else
-		make_options(argv[NA(argc)], &res->startup_options);
+		make_options(argv[NA(argc)], &res->startup_options, ctx->cmd->drbdsetup_ctx);
 	argv[NA(argc)] = 0;
 
 	rv = m_system_ex(argv, SLEEPS_FOREVER, res->name);
@@ -2130,6 +2175,18 @@ static bool set_ignore_flag(struct connection * const conn, checks check, bool i
 	return true;
 }
 
+static struct connection *find_conn_on_cmdline(struct connections *connections)
+{
+	struct connection *conn;
+
+	for_each_connection(conn, connections) {
+		if (conn->on_cmdline == 1)
+			return conn;
+	}
+
+	return NULL;
+}
+
 int ctx_by_name(struct cfg_ctx *ctx, const char *id, checks check)
 {
 	struct d_resource *res;
@@ -2139,6 +2196,7 @@ int ctx_by_name(struct cfg_ctx *ctx, const char *id, checks check)
 	char *vol_id;
 	char *res_name, *conn_or_hostname;
 	unsigned vol_nr = ~0U;
+	int connections_found = 0;
 
 	res_name = input;
 	vol_id = strrchr(input, '/');
@@ -2186,6 +2244,7 @@ int ctx_by_name(struct cfg_ctx *ctx, const char *id, checks check)
 				}
 
 				if (conn->peer == hi) {
+					connections_found++;
 					conn->on_cmdline = 1;
 					if (check == CTX_FIRST)
 						goto found;
@@ -2202,6 +2261,7 @@ int ctx_by_name(struct cfg_ctx *ctx, const char *id, checks check)
 				struct d_option *opt;
 				opt = find_opt(&conn->net_options, "_name");
 				if (opt && !strcmp(opt->value, conn_or_hostname)) {
+					connections_found++;
 					if (check == CTX_FIRST)
 						goto found;
 
@@ -2227,7 +2287,8 @@ int ctx_by_name(struct cfg_ctx *ctx, const char *id, checks check)
 	if (check != SETUP_MULTI)
 		return 0;
 
-	if (0) {
+	if (connections_found == 1) {
+		conn = find_conn_on_cmdline(&res->connections);
 found:
 		if (conn->ignore) {
 			err("Connection '%s' has the ignore flag set\n",
@@ -2450,7 +2511,7 @@ static int adm_wait_ci(const struct cfg_ctx *ctx)
 		argv[NA(argc)] = drbdsetup;
 		argv[NA(argc)] = "wait-connect-resource";
 		argv[NA(argc)] = res->name;
-		make_options(argv[NA(argc)], &res->startup_options);
+		make_options(argv[NA(argc)], &res->startup_options, ctx->cmd->drbdsetup_ctx);
 		argv[NA(argc)] = 0;
 
 		m__system(argv, RETURN_PID, res->name, &pids[i++], NULL, NULL);
@@ -2729,15 +2790,7 @@ void verify_ips(struct d_resource *res)
 	}
 }
 
-static char *conf_file[] = {
-	DRBD_CONFIG_DIR "/drbd-90.conf",
-	DRBD_CONFIG_DIR "/drbd-84.conf",
-	DRBD_CONFIG_DIR "/drbd-83.conf",
-	DRBD_CONFIG_DIR "/drbd-82.conf",
-	DRBD_CONFIG_DIR "/drbd-08.conf",
-	DRBD_CONFIG_DIR "/drbd.conf",
-	0
-};
+extern char *conf_file[];
 
 int pushd(const char *path)
 {
@@ -3171,6 +3224,10 @@ char *config_file_from_arg(char *arg)
 void assign_default_config_file(void)
 {
 	int i;
+
+	generate_conf_file_locations();
+		/* On Windows this is dynamic, NOOP on Linux */
+
 	for (i = 0; conf_file[i]; i++) {
 		yyin = fopen(conf_file[i], "r");
 		if (yyin) {
@@ -3455,7 +3512,9 @@ int main(int argc, char **argv)
 						exit(E_USAGE);
 					}
 				}
-				if (cmd->vol_id_required && !ctx.vol && STAILQ_FIRST(&ctx.res->me->volumes)->implicit)
+				if (cmd->vol_id_required && !ctx.vol
+				&& !STAILQ_EMPTY(&ctx.res->me->volumes)
+				&& STAILQ_FIRST(&ctx.res->me->volumes)->implicit)
 					ctx.vol = STAILQ_FIRST(&ctx.res->me->volumes);
 				if (cmd->vol_id_required && !ctx.vol) {
 					err("%s requires a specific volume id, but none is specified.\n"
