@@ -34,6 +34,7 @@
 #include "drbd_nla.h"
 #include "drbdtool_common.h"
 #include <linux/genl_magic_func.h>
+#include "drbd_protocol.h"
 #include "drbd_strings.h"
 #include "drbdsetup_colors.h"
 
@@ -53,6 +54,7 @@ static const char *action_change = "change";
 static const char *action_destroy = "destroy";
 static const char *action_call = "call";
 static const char *action_response = "response";
+static const char *action_rename = "rename";
 
 static const char *object_resource = "resource";
 static const char *object_device = "device";
@@ -61,10 +63,12 @@ static const char *object_peer_device = "peer-device";
 static const char *object_helper = "helper";
 static const char *object_path = "path";
 
+bool initial_state = true; /* receiving new data in "exists" messages */
+bool receive_update = false; /* receiving updates in "exists" messages */
 void *all_resources;
 struct resources_list *update_resources;
 
-static int apply_event(const char *prefix, struct genl_info *info, bool initial_state);
+static int apply_event(const char *prefix, struct genl_info *info);
 
 static int resource_obj_cmp(const void *a, const void *b)
 {
@@ -383,6 +387,7 @@ static struct resources_list *deep_copy_resource(struct resources_list *old_reso
 	new_resource->res_opts = nla_copy(old_resource->res_opts);
 	new_resource->info = old_resource->info;
 	new_resource->statistics = old_resource->statistics;
+	new_resource->rename_info = old_resource->rename_info;
 
 	for (old_device = old_resource->devices; old_device; old_device = old_device->next) {
 		struct devices_list *new_device;
@@ -472,6 +477,30 @@ static int count_up_to_date_replicas(struct resources_list *resource, struct dev
 	return up_to_date_replicas;
 }
 
+static bool have_stable_up_to_date_replicas(struct resources_list *resource, struct devices_list *device)
+{
+	struct connections_list *connection;
+	struct peer_devices_list *peer_device;
+
+	if (device->info.dev_disk_state == D_UP_TO_DATE) {
+		/* Technically this device may be unstable due to a primary
+		 * peer, but that is handled separately, so we do not check it
+		 * here. */
+		return true;
+	}
+
+	for (connection = resource->connections; connection; connection = connection->next) {
+		for (peer_device = connection->peer_devices; peer_device; peer_device = peer_device->next) {
+			if (peer_device->ctx.ctx_volume == device->ctx.ctx_volume &&
+					peer_device->info.peer_disk_state == D_UP_TO_DATE &&
+					peer_device->statistics.peer_dev_uuid_flags & UUID_FLAG_STABLE)
+				return true;
+		}
+	}
+
+	return false;
+}
+
 static bool have_primary(struct resources_list *resource)
 {
 	struct connections_list *connection;
@@ -500,6 +529,7 @@ static struct promotion_info compute_promotion_info(struct resources_list *resou
 	int up_to_date_devices = 0;
 	bool all_devices_have_disk = true;
 	int min_up_to_date_replicas = -1;
+	bool all_devices_stable_up_to_date = true;
 	int all_local_devices_score;
 	int local_devices_score;
 	int replicas_score;
@@ -513,6 +543,7 @@ static struct promotion_info compute_promotion_info(struct resources_list *resou
 
 	for (device = resource->devices; device; device = device->next) {
 		int up_to_date_replicas = count_up_to_date_replicas(resource, device);
+		bool stable_up_to_date_replicas = have_stable_up_to_date_replicas(resource, device);
 		enum drbd_disk_state disk_state = device->info.dev_disk_state;
 
 		device_count++;
@@ -527,12 +558,15 @@ static struct promotion_info compute_promotion_info(struct resources_list *resou
 
 		if (min_up_to_date_replicas == -1 || up_to_date_replicas < min_up_to_date_replicas)
 			min_up_to_date_replicas = up_to_date_replicas;
+
+		if (!stable_up_to_date_replicas)
+			all_devices_stable_up_to_date = false;
 	}
 
 	if (min_up_to_date_replicas == -1 || min_up_to_date_replicas == 0)
 		return info;
 
-	info.may_promote = !have_primary(resource) && (up_to_date_devices == device_count);
+	info.may_promote = !have_primary(resource) && all_devices_stable_up_to_date;
 
 	all_local_devices_score = all_devices_have_disk ? 1 : 0;
 	local_devices_score = min(up_to_date_devices, 99);
@@ -548,11 +582,12 @@ static struct promotion_info compute_promotion_info(struct resources_list *resou
 static void print_resource_changes(const char *prefix, const char *action_new, struct resources_list *old_resource, struct resources_list *new_resource)
 {
 	struct promotion_info new_promotion_info;
-	struct promotion_info old_promotion_info;
+	struct promotion_info old_promotion_info = { 0 };
 	bool role_changed;
 	bool info_changed;
 	bool statistics_changed;
 	bool promotion_info_changed;
+	bool renamed;
 
 	new_promotion_info = compute_promotion_info(new_resource);
 
@@ -574,23 +609,56 @@ static void print_resource_changes(const char *prefix, const char *action_new, s
 		promotion_info_changed = true;
 	}
 
+	renamed = new_resource->rename_info.res_new_name_len > 0;
+
+	if (renamed) {
+		printf("%s%s %s name:%s new_name:%s\n", prefix, action_rename, object_resource,
+				new_resource->name, new_resource->rename_info.res_new_name);
+		free(new_resource->name);
+		new_resource->name = strdup(new_resource->rename_info.res_new_name);
+		return;
+	}
+
 	if (!role_changed && !info_changed && !statistics_changed && !promotion_info_changed)
 		return;
 
 	printf("%s%s ", prefix, old_resource ? action_change : action_new);
 	printf("%s name:%s", object_resource, new_resource->name);
-	if (role_changed)
-		printf(" role:%s%s%s",
-				ROLE_COLOR_STRING(new_resource->info.res_role, 1));
-	if (info_changed)
-		printf(" suspended:%s",
-				susp_str(&new_resource->info));
-	if (statistics_changed)
+	if (role_changed || opt_fullch) {
+		if (opt_diff)
+			DIFF_COLOR(old_resource, "role",
+					ROLE_COLOR_STRING(old_resource->info.res_role, 1),
+					ROLE_COLOR_STRING(new_resource->info.res_role, 1));
+		else
+			printf(" role:%s%s%s", ROLE_COLOR_STRING(new_resource->info.res_role, 1));
+	}
+	if (info_changed || opt_fullch) {
+		if (opt_diff)
+			printf(" suspended:%s->%s",
+					old_resource ? susp_str(&old_resource->info) : UNKNOWN_STRING,
+					susp_str(&new_resource->info));
+		else
+			printf(" suspended:%s", susp_str(&new_resource->info));
+	}
+	if (statistics_changed || opt_fullch)
 		print_resource_statistics(0, old_resource ? &old_resource->statistics : NULL,
 				&new_resource->statistics, nowrap_printf);
-	if (promotion_info_changed) {
-		printf(" may_promote:%s", new_promotion_info.may_promote ? "yes" : "no");
-		printf(" promotion_score:%d", new_promotion_info.promotion_score);
+	if (promotion_info_changed || opt_fullch) {
+		if (opt_diff) {
+			char *old_maypromote = UNKNOWN_STRING;
+			if (old_resource)
+				old_maypromote = old_promotion_info.may_promote ? "yes" : "no";
+
+			printf(" may_promote:%s->%s promotion_score:%d->%d",
+			       old_maypromote,
+			       new_promotion_info.may_promote ? "yes" : "no",
+			       old_resource ? old_promotion_info.promotion_score : 0,
+			       new_promotion_info.promotion_score);
+		}
+		else
+			printf(" may_promote:%s promotion_score:%d",
+			       new_promotion_info.may_promote ? "yes" : "no",
+			       new_promotion_info.promotion_score);
 	}
 	printf("\n");
 }
@@ -611,14 +679,56 @@ static void print_device_changes(const char *prefix, const char *action_new, con
 
 	printf("%s%s ", prefix, old_device ? action_change : action_new);
 	printf("%s name:%s volume:%u minor:%u", object_device, resource_name, new_device->ctx.ctx_volume, new_device->minor);
-	if (info_changed) {
+	if (info_changed || opt_fullch) {
 		bool intentional = new_device->info.is_intentional_diskless == 1;
-		printf(" disk:%s%s%s",
-				DISK_COLOR_STRING(new_device->info.dev_disk_state, intentional, true));
-		printf(" client:%s", intentional_diskless_str(&new_device->info));
-		printf(" quorum:%s", new_device->info.dev_has_quorum ? "yes" : "no");
+		if (opt_diff) {
+			bool old_intentional = 0; /* only used if we have an old_device */
+
+			if (!old_device || strcmp(backing_dev_str(&old_device->info), backing_dev_str(&new_device->info)) != 0) {
+				const char *old_dev_backing_dev = UNKNOWN_STRING;
+				if (old_device)
+					old_dev_backing_dev = backing_dev_str(&old_device->info);
+
+				printf(" backing_dev:%s->%s",
+						old_dev_backing_dev,
+						backing_dev_str(&new_device->info));
+			} else {
+				printf(" backing_dev:%s", backing_dev_str(&new_device->info));
+			}
+
+			if (old_device)
+				old_intentional = (old_device->info.is_intentional_diskless == 1);
+
+			DIFF_COLOR(old_device, "disk",
+					DISK_COLOR_STRING(old_device->info.dev_disk_state, old_intentional, true),
+					DISK_COLOR_STRING(new_device->info.dev_disk_state, intentional, true));
+
+			if (!old_device || old_intentional != intentional)
+				printf(" client:%s->%s",
+						old_device ? intentional_diskless_str(&old_device->info) : UNKNOWN_STRING,
+						intentional_diskless_str(&new_device->info));
+			else
+				printf(" client:%s", intentional_diskless_str(&new_device->info));
+
+			if (!old_device || old_device->info.dev_has_quorum != new_device->info.dev_has_quorum) {
+				char *old_dev_has_quorum = UNKNOWN_STRING;
+				if (old_device)
+					old_dev_has_quorum = old_device->info.dev_has_quorum ? "yes" : "no",
+
+				printf(" quorum:%s->%s",
+						old_dev_has_quorum,
+						new_device->info.dev_has_quorum ? "yes" : "no");
+			} else
+				printf(" quorum:%s", new_device->info.dev_has_quorum ? "yes" : "no");
+		} else {
+			printf(" backing_dev:%s", backing_dev_str(&new_device->info));
+			printf(" disk:%s%s%s",
+					DISK_COLOR_STRING(new_device->info.dev_disk_state, intentional, true));
+			printf(" client:%s", intentional_diskless_str(&new_device->info));
+			printf(" quorum:%s", new_device->info.dev_has_quorum ? "yes" : "no");
+		}
 	}
-	if (statistics_changed) {
+	if (statistics_changed || opt_fullch) {
 		print_device_statistics(0, old_device ? &old_device->statistics : NULL,
 				&new_device->statistics, nowrap_printf);
 	}
@@ -631,6 +741,11 @@ static void print_peer_device_changes(const char *prefix, const char *action_new
 	bool disk_changed;
 	bool resync_suspended_changed;
 	bool statistics_changed;
+	bool intentional = new_peer_device->info.peer_is_intentional_diskless == 1;
+	bool old_intentional = 0;
+
+	if (old_peer_device)
+		old_intentional = old_peer_device->info.peer_is_intentional_diskless == 1;
 
 	repl_state_changed = !old_peer_device || new_peer_device->info.peer_repl_state != old_peer_device->info.peer_repl_state;
 	disk_changed = !old_peer_device || new_peer_device->info.peer_disk_state != old_peer_device->info.peer_disk_state;
@@ -646,22 +761,46 @@ static void print_peer_device_changes(const char *prefix, const char *action_new
 
 	printf("%s%s ", prefix, old_peer_device ? action_change : action_new);
 	printf("%s name:%s peer-node-id:%u conn-name:%s volume:%u", object_peer_device, resource_name, new_peer_device->ctx.ctx_peer_node_id, new_peer_device->ctx.ctx_conn_name, new_peer_device->ctx.ctx_volume);
-	if (repl_state_changed) {
-		printf(" replication:%s%s%s",
-				REPL_COLOR_STRING(new_peer_device->info.peer_repl_state));
+
+	if (repl_state_changed || opt_fullch) {
+		if (opt_diff)
+			DIFF_COLOR(old_peer_device, "replication",
+					REPL_COLOR_STRING(old_peer_device->info.peer_repl_state),
+					REPL_COLOR_STRING(new_peer_device->info.peer_repl_state));
+		else
+			printf(" replication:%s%s%s",
+			       REPL_COLOR_STRING(new_peer_device->info.peer_repl_state));
 	}
-	if (disk_changed) {
-		bool intentional = new_peer_device->info.peer_is_intentional_diskless == 1;
-		printf(" peer-disk:%s%s%s",
-				DISK_COLOR_STRING(new_peer_device->info.peer_disk_state, intentional,  false));
-		printf(" peer-client:%s", peer_intentional_diskless_str(&new_peer_device->info));
+	if (disk_changed || opt_fullch || intentional != old_intentional) {
+		if (opt_diff) {
+
+			DIFF_COLOR(old_peer_device, "peer-disk",
+					DISK_COLOR_STRING(old_peer_device->info.peer_disk_state, old_intentional, false),
+					DISK_COLOR_STRING(new_peer_device->info.peer_disk_state, intentional, false));
+
+			if (!old_peer_device || old_intentional != intentional)
+				printf(" peer-client:%s->%s",
+				       old_peer_device ? peer_intentional_diskless_str(&old_peer_device->info) : UNKNOWN_STRING,
+				       peer_intentional_diskless_str(&new_peer_device->info));
+			else
+				printf(" peer-client:%s", peer_intentional_diskless_str(&new_peer_device->info));
+		} else {
+			printf(" peer-disk:%s%s%s",
+			       DISK_COLOR_STRING(new_peer_device->info.peer_disk_state, intentional,  false));
+			printf(" peer-client:%s", peer_intentional_diskless_str(&new_peer_device->info));
+		}
 	}
-	if (resync_suspended_changed) {
-		printf(" resync-suspended:%s",
-				resync_susp_str(&new_peer_device->info));
+	if (resync_suspended_changed || opt_fullch) {
+		if (opt_diff)
+			printf(" resync-suspended:%s->%s",
+			       old_peer_device ? resync_susp_str(&old_peer_device->info) : UNKNOWN_STRING,
+			       resync_susp_str(&new_peer_device->info));
+		else
+			printf(" resync-suspended:%s",
+			       resync_susp_str(&new_peer_device->info));
 	}
 
-	if (statistics_changed) {
+	if (statistics_changed || opt_fullch) {
 		print_peer_device_statistics(0, old_peer_device ? &old_peer_device->statistics : NULL,
 				&new_peer_device->statistics, nowrap_printf);
 	}
@@ -687,9 +826,18 @@ static void print_path_changes(const char *prefix, const char *action_new, const
 			object_path, resource_name,
 			new_path->ctx.ctx_peer_node_id, new_path->ctx.ctx_conn_name,
 			my_addr, peer_addr);
-	if (established_changed) {
-		printf(" established:%s",
-				new_path->info.path_established ? "yes" : "no");
+	if (established_changed || opt_fullch) {
+		if (opt_diff) {
+			char *old_established = UNKNOWN_STRING;
+			if (old_path)
+				old_established = old_path->info.path_established ? "yes" : "no";
+			printf(" established:%s->%s",
+			       old_established,
+			       new_path->info.path_established ? "yes" : "no");
+		}
+		else
+			printf(" established:%s",
+			       new_path->info.path_established ? "yes" : "no");
 	}
 	printf("\n");
 }
@@ -711,15 +859,25 @@ static void print_connection_changes(const char *prefix, const char *action_new,
 
 	printf("%s%s ", prefix, old_connection ? action_change : action_new);
 	printf("%s name:%s peer-node-id:%u conn-name:%s", object_connection, resource_name, new_connection->ctx.ctx_peer_node_id, new_connection->ctx.ctx_conn_name);
-	if (connection_state_changed) {
-		printf(" connection:%s%s%s",
-				CONN_COLOR_STRING(new_connection->info.conn_connection_state));
+	if (connection_state_changed || opt_fullch) {
+		if (opt_diff)
+			DIFF_COLOR(old_connection, "connection",
+					CONN_COLOR_STRING(old_connection->info.conn_connection_state),
+					CONN_COLOR_STRING(new_connection->info.conn_connection_state));
+		else
+			printf(" connection:%s%s%s",
+			       CONN_COLOR_STRING(new_connection->info.conn_connection_state));
 	}
-	if (role_changed) {
-		printf(" role:%s%s%s",
-				ROLE_COLOR_STRING(new_connection->info.conn_role, 0));
+	if (role_changed || opt_fullch) {
+		if (opt_diff)
+			DIFF_COLOR(old_connection, "role",
+					ROLE_COLOR_STRING(old_connection->info.conn_role, 0),
+					ROLE_COLOR_STRING(new_connection->info.conn_role, 0));
+		else
+			printf(" role:%s%s%s",
+			       ROLE_COLOR_STRING(new_connection->info.conn_role, 0));
 	}
-	if (statistics_changed) {
+	if (statistics_changed || opt_fullch) {
 		print_connection_statistics(0, old_connection ? &old_connection->statistics : NULL,
 				&new_connection->statistics, nowrap_printf);
 	}
@@ -896,7 +1054,6 @@ int print_event(struct drbd_cmd *cm, struct genl_info *info, void *u_ptr)
 {
 	static uint32_t last_seq;
 	static bool last_seq_known;
-	static bool initial_state = true;
 	static struct nlmsg_entry *stored_messages = NULL;
 
 	struct drbd_notification_header nh = { .nh_type = -1U };
@@ -919,38 +1076,38 @@ int print_event(struct drbd_cmd *cm, struct genl_info *info, void *u_ptr)
 		return 0;
 	action = nh.nh_type & ~NOTIFY_FLAGS;
 
-	if (opt_now && action != NOTIFY_EXISTS)
-		return 0;
-
 	err = format_timestamp(timestamp_prefix);
 	if (err)
 		exit(20);
 
 	if (info->genlhdr->cmd == DRBD_INITIAL_STATE_DONE) {
 		struct nlmsg_entry *entry, *next_entry;
-		initial_state = false;
-		printf("%s%s -\n", timestamp_prefix, action_exists);
+
+		if (initial_state)
+			printf("%s%s -\n", timestamp_prefix, action_exists);
 		fflush(stdout);
-		if (opt_now)
-			return -1;
+
+		initial_state = false;
+		receive_update = false;
 
 		/* now apply stored messages */
 		for (entry = stored_messages; entry; entry = next_entry) {
+			struct nlattr *tla[ARRAY_SIZE(drbd_tla_nl_policy)];
 			struct genl_info stored_info = {
 				.seq = entry->nlh->nlmsg_seq,
 				.nlhdr = entry->nlh,
 				.genlhdr = nlmsg_data(entry->nlh),
 				.userhdr = genlmsg_data(nlmsg_data(entry->nlh)),
-				.attrs = global_attrs,
+				.attrs = tla,
 			};
 
-			err = drbd_tla_parse(entry->nlh);
+			err = drbd_tla_parse(stored_info.attrs, entry->nlh);
 			if (err) {
 				fprintf(stderr, "drbd_tla_parse() failed");
 				return 1;
 			}
 
-			err = apply_event(timestamp_prefix, &stored_info, false);
+			err = apply_event(timestamp_prefix, &stored_info);
 			if (err)
 				return err;
 
@@ -958,6 +1115,7 @@ int print_event(struct drbd_cmd *cm, struct genl_info *info, void *u_ptr)
 			free(entry->nlh);
 			free(entry);
 		}
+		stored_messages = NULL;
 
 		return 0;
 	}
@@ -974,7 +1132,7 @@ int print_event(struct drbd_cmd *cm, struct genl_info *info, void *u_ptr)
 		last_seq = info->nlhdr->nlmsg_seq;
 		last_seq_known = true;
 
-		if (initial_state) {
+		if (initial_state || receive_update) {
 			/* store message until initial state is finished */
 			struct nlmsg_entry *entry, **previous_next = &stored_messages;
 			for (entry = stored_messages; entry; entry = entry->next)
@@ -984,10 +1142,10 @@ int print_event(struct drbd_cmd *cm, struct genl_info *info, void *u_ptr)
 		}
 	}
 
-	return apply_event(timestamp_prefix, info, initial_state);
+	return apply_event(timestamp_prefix, info);
 }
 
-static int apply_event(const char *prefix, struct genl_info *info, bool initial_state)
+static int apply_event(const char *prefix, struct genl_info *info)
 {
 	int err;
 	struct drbd_notification_header nh = { .nh_type = -1U };
@@ -1006,6 +1164,9 @@ static int apply_event(const char *prefix, struct genl_info *info, bool initial_
 		return 0;
 	action = nh.nh_type & ~NOTIFY_FLAGS;
 
+	if (action == NOTIFY_EXISTS && receive_update)
+		action = NOTIFY_CHANGE; /* exists messages are actually updates */
+
 	err = drbd_cfg_context_from_attrs(&ctx, info);
 	if (err)
 		return 0;
@@ -1035,6 +1196,9 @@ static int apply_event(const char *prefix, struct genl_info *info, bool initial_
 		store_update_resource(new_resource);
 	}
 
+	new_resource->rename_info.res_new_name[0] = '\0';
+	new_resource->rename_info.res_new_name_len = 0;
+
 	switch (action) {
 	case NOTIFY_EXISTS:
 	case NOTIFY_CREATE:
@@ -1162,6 +1326,13 @@ static int apply_event(const char *prefix, struct genl_info *info, bool initial_
 				action == NOTIFY_RESPONSE, &helper_info);
 		break;
 	}
+        case NOTIFY_RENAME:
+		switch(info->genlhdr->cmd) {
+			case DRBD_RESOURCE_STATE:
+				rename_resource_info_from_attrs(&new_resource->rename_info, info);
+				break;
+                }
+                break;
 	default:
 		dbg(1, "unknown notification type %d\n", action);
 		goto out;
@@ -1195,3 +1366,22 @@ out:
 	fflush(stdout);
 	return 0;
 }
+
+/* New "initial state" is incoming - treat "exists" messages as updates. */
+void events2_prepare_update()
+{
+	receive_update = true;
+}
+
+#ifndef HAVE___FREE_FN_T
+	/* Windows cygwin hasn't defined this, for example */
+typedef void (*__free_fn_t) (void *__nodep);
+#endif
+
+/* Drop all data and start again with new initial state. */
+void events2_reset()
+{
+	tdestroy(all_resources, (__free_fn_t) free_resource);
+	all_resources = NULL;
+	initial_state = true;
+}
